# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import

import pytest
from aca import Automaton

def test_has_pattern():
    automaton = Automaton()
    automaton.add('he')
    automaton.add('she')
    automaton.add('us')

    assert automaton.has_pattern('he')
    assert automaton.has_pattern('she')
    assert automaton.has_pattern('us')
    assert not automaton.has_pattern('they')
    assert not automaton.has_pattern('e')
    assert not automaton.has_pattern('use')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import

from aca import Automaton, Match


def test_map_interface():
    auto = Automaton()
    auto['us'] = 'USA'
    auto['her'] ='EUROPE'
    assert auto['us'] == 'USA'
    assert auto['her'] == 'EUROPE'
    matches = auto.get_matches('usher')
    assert len(matches) == 2
    assert matches[0].label == 'USA'
    assert matches[1].label == 'EUROPE'

    assert 'us' in auto
    assert 'his' not in auto
    #assert list(auto.keys()) == ['her', 'us']
    #assert list(auto.values()) == ['EUROPE', 'USA']
    #assert list(k for k in auto) == list(auto.keys())



# -*- coding: utf-8 -*-
"""
Example/test script that detects names from a longer text
"""
from __future__ import unicode_literals, print_function, absolute_import
import os
from tempfile import TemporaryDirectory
from aca import Automaton, Match

KEY = 'everything\non\nseparate\nlines'
VAL = 'YES'

def test_names():
    auto = Automaton()
    auto[KEY] = VAL
    auto.update_automaton()

    with TemporaryDirectory() as tmpdir:
        fnm = os.path.join(tmpdir, 'test.aca')
        auto.save_to_file(fnm)
        auto2 = Automaton()
        auto2.load_from_file(fnm)

    assert auto2[KEY] == VAL




# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import
from aca.aca_cpp import Match, remove_overlaps


def matches(positions):
    return [Match(start, end) for start, end in positions]

def test_empty_list():
    assert [] == remove_overlaps([])


def test_single_span():
    assert [Match(1, 2)] == remove_overlaps([Match(1, 2)])


def test_nonoverlapping_spans():
    test_input = matches([(1, 2), (2, 3), (4, 5)])
    expected = test_input.copy()
    assert expected == remove_overlaps(test_input)

def test_two_spans_overlap():
    test_input = matches([(1, 2), (1, 3)])
    expected = matches([(1, 3)])
    assert expected == remove_overlaps(test_input)


def test_three_spans_overlap():
    test_input = matches([(1, 3), (2, 5), (4, 5)])
    expected = matches([(1, 3), (4, 5)])
    assert expected == remove_overlaps(test_input)


def test_two_overlapping_clusters():
    test_input = matches([(1, 3), (1, 2), (5, 7), (5, 6)])
    expected = matches([(1, 3), (5, 7)])
    assert expected == remove_overlaps(test_input)


def test_one_large_segment():
    test_input = matches([(1, 2), (2, 3), (4, 5), (4, 8), (5, 6), (6, 9), (7, 9), (2, 7), (1, 10)])
    expected = matches([(1, 10)])
    assert expected == remove_overlaps(test_input)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import

# create a new AC automaton
from aca import Automaton
automaton = Automaton()

# instead of plain strings, you can also use lists of tokens
names = [
    (['Yuri', 'Artyukhin'], 'developer'),
    (['Tom', 'Anderson', 'Jr'], 'designer'),
]
automaton.add_all(names)

# you can add an item like this as well
automaton[['Tom', 'Anderson']] = 'manager'


# if you are not using plain strings, make sure you tokenize the text as well
text = 'Tom Anderson Jr and Yuri Artyukhin work on my project'.split()

print ('matches that maximize the number of matched words')
for match in automaton.get_matches(text):
    print (match.start, match.end, match.elems, match.label)


print ('all matches')
for match in automaton.get_matches(text, exclude_overlaps=False):
    print (match.start, match.end, match.elems, match.label)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import

# create a new AC automaton
from aca import Automaton
automaton = Automaton()

# add a dictionary of words to the automaton
painkillers = ['paracetamol', 'ibuprofen', 'hydrocloride']
automaton.add_all(painkillers)

# match the dictionary on a text
text = 'paracetamol and hydrocloride are a medications to relieve pain and fever. paracetamol is less efficient than ibuprofen'

for match in automaton.get_matches(text):
    print (match.start, match.end, match.elems)
